@@ -15,7 +15,7 @@
  *****************************************************************************/
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 308150 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 316874 $")
 
 #include "asterisk/io.h"
 #include "asterisk/lock.h"
@@ -522,10 +522,10 @@ int ooGetLocalIPAddress(char * pIPAddrs)
    ret = gethostname(hostname, 100);
    if(ret == 0)
    {
-      if (!(hp = ast_gethostbyname(hostname, &phost))) {
+      if ((hp = ast_gethostbyname(hostname, &phost))) {
 	  		struct in_addr i;
 			memcpy(&i, hp->h_addr, sizeof(i));
-			  strcpy(pIPAddrs, (ast_inet_ntoa(i) == NULL) ? "127.0.0.1" : ast_inet_ntoa(i));
+			strcpy(pIPAddrs, (ast_inet_ntoa(i) == NULL) ? "127.0.0.1" : ast_inet_ntoa(i));
       } else {
          return -1;
       }
