@@ -25,9 +25,14 @@
  * \ingroup applications
  */
 
+/*** MODULEINFO
+	<support_level>deprecated</support_level>
+	<replacement>app_stack (GoSub)</replacement>
+ ***/
+
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 295866 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 328446 $")
 
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
