@@ -29,11 +29,12 @@
 
 /*** MODULEINFO
 	<depend>alsa</depend>
+	<support_level>extended</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 278132 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 328209 $")
 
 #include <fcntl.h>
 #include <sys/ioctl.h>
@@ -57,13 +58,14 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 278132 $")
 #include "asterisk/musiconhold.h"
 #include "asterisk/poll-compat.h"
 
-/*! Global jitterbuffer configuration - by default, jb is disabled */
+/*! Global jitterbuffer configuration - by default, jb is disabled
+ *  \note Values shown here match the defaults shown in alsa.conf.sample */
 static struct ast_jb_conf default_jbconf = {
 	.flags = 0,
-	.max_size = -1,
-	.resync_threshold = -1,
-	.impl = "",
-	.target_extra = -1,
+	.max_size = 200,
+	.resync_threshold = 1000,
+	.impl = "fixed",
+	.target_extra = 40,
 };
 static struct ast_jb_conf global_jbconf;
 
@@ -383,7 +385,6 @@ static int alsa_write(struct ast_channel *chan, struct ast_frame *f)
 	static char sizbuf[8000];
 	static int sizpos = 0;
 	int len = sizpos;
-	int pos;
 	int res = 0;
 	/* size_t frames = 0; */
 	snd_pcm_state_t state;
@@ -397,7 +398,6 @@ static int alsa_write(struct ast_channel *chan, struct ast_frame *f)
 	} else {
 		memcpy(sizbuf + sizpos, f->data.ptr, f->datalen);
 		len += f->datalen;
-		pos = 0;
 		state = snd_pcm_state(alsa.ocard);
 		if (state == SND_PCM_STATE_XRUN)
 			snd_pcm_prepare(alsa.ocard);
