@@ -1,7 +1,7 @@
 /*
  * Asterisk -- An open source telephony toolkit.
  *
- * Copyright (C) 2007-2008, Dwayne M. Hubbard 
+ * Copyright (C) 2007-2008, Digium, Inc.
  *
  * Dwayne M. Hubbard <dhubbard@digium.com>
  *
@@ -15,8 +15,9 @@
  * the GNU General Public License Version 2. See the LICENSE file
  * at the top of the source tree.
  */
-/*! \file
- *
+
+/*!
+ * \file
  * \brief Maintain a container of uniquely-named taskprocessor threads that can be shared across modules.
  *
  * \author Dwayne Hubbard <dhubbard@digium.com>
@@ -24,10 +25,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 157042 $")
-
-#include <signal.h>
-#include <sys/time.h>
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 248226 $")
 
 #include "asterisk/_private.h"
 #include "asterisk/module.h"
@@ -37,11 +35,13 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 157042 $")
 #include "asterisk/taskprocessor.h"
 
 
-/*! \brief tps_task structure is queued to a taskprocessor
+/*!
+ * \brief tps_task structure is queued to a taskprocessor
  *
  * tps_tasks are processed in FIFO order and freed by the taskprocessing
  * thread after the task handler returns.  The callback function that is assigned
- * to the execute() function pointer is responsible for releasing datap resources if necessary. */
+ * to the execute() function pointer is responsible for releasing datap resources if necessary.
+ */
 struct tps_task {
 	/*! \brief The execute() task callback function pointer */
 	int (*execute)(void *datap);
@@ -62,7 +62,7 @@ struct tps_taskprocessor_stats {
 /*! \brief A ast_taskprocessor structure is a singleton by name */
 struct ast_taskprocessor {
 	/*! \brief Friendly name of the taskprocessor */
-	char *name;
+	const char *name;
 	/*! \brief Thread poll condition */
 	ast_cond_t poll_cond;
 	/*! \brief Taskprocessor thread */
@@ -84,10 +84,10 @@ struct ast_taskprocessor {
 /*! \brief tps_singletons is the astobj2 container for taskprocessor singletons */
 static struct ao2_container *tps_singletons;
 
-/*! \brief CLI 'taskprocessor ping <blah>' operation requires a ping condition */
+/*! \brief CLI <example>taskprocessor ping &lt;blah&gt;</example> operation requires a ping condition */
 static ast_cond_t cli_ping_cond;
 
-/*! \brief CLI 'taskprocessor ping <blah>' operation requires a ping condition lock */
+/*! \brief CLI <example>taskprocessor ping &lt;blah&gt;</example> operation requires a ping condition lock */
 AST_MUTEX_DEFINE_STATIC(cli_ping_cond_lock);
 
 /*! \brief The astobj2 hash callback for taskprocessors */
@@ -101,7 +101,7 @@ static void *tps_processing_function(void *data);
 /*! \brief Destroy the taskprocessor when its refcount reaches zero */
 static void tps_taskprocessor_destroy(void *tps);
 
-/*! \brief CLI 'taskprocessor ping <blah>' handler function */
+/*! \brief CLI <example>taskprocessor ping &lt;blah&gt;</example> handler function */
 static int tps_ping_handler(void *datap);
 
 /*! \brief Remove the front task off the taskprocessor queue */
@@ -114,7 +114,7 @@ static char *cli_tps_ping(struct ast_cli_entry *e, int cmd, struct ast_cli_args
 static char *cli_tps_report(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a);
 
 static struct ast_cli_entry taskprocessor_clis[] = {
-	AST_CLI_DEFINE(cli_tps_ping, "Ping a named task processors"),
+	AST_CLI_DEFINE(cli_tps_ping, "Ping a named task processor"),
 	AST_CLI_DEFINE(cli_tps_report, "List instantiated task processors and statistics"),
 };
 
@@ -189,16 +189,16 @@ static int tps_ping_handler(void *datap)
 static char *cli_tps_ping(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 	struct timeval begin, end, delta;
-	char *name;
+	const char *name;
 	struct timeval when;
 	struct timespec ts;
 	struct ast_taskprocessor *tps = NULL;
 
 	switch (cmd) {
 	case CLI_INIT:
-		e->command = "core taskprocessor ping";
+		e->command = "core ping taskprocessor";
 		e->usage = 
-			"Usage: core taskprocessor ping <taskprocessor>\n"
+			"Usage: core ping taskprocessor <taskprocessor>\n"
 			"	Displays the time required for a task to be processed\n";
 		return NULL;
 	case CLI_GENERATE:
@@ -366,7 +366,7 @@ static void tps_taskprocessor_destroy(void *tps)
 		ast_free(t->stats);
 		t->stats = NULL;
 	}
-	ast_free(t->name);
+	ast_free((char *) t->name);
 }
 
 /* pop the front task and return it */
@@ -404,7 +404,7 @@ const char *ast_taskprocessor_name(struct ast_taskprocessor *tps)
 /* Provide a reference to a taskprocessor.  Create the taskprocessor if necessary, but don't
  * create the taskprocessor if we were told via ast_tps_options to return a reference only 
  * if it already exists */
-struct ast_taskprocessor *ast_taskprocessor_get(char *name, enum ast_tps_options create)
+struct ast_taskprocessor *ast_taskprocessor_get(const char *name, enum ast_tps_options create)
 {
 	struct ast_taskprocessor *p, tmp_tps = {
 		.name = name,
