@@ -23,9 +23,13 @@
  * \brief pthread timing interface
  */
 
+/*** MODULEINFO
+	<support_level>extended</support_level>
+ ***/
+
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 278465 $");
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 328209 $");
 
 #include <math.h>
 #include <sys/select.h>
