@@ -25,6 +25,7 @@
 
 /*** MODULEINFO
 	<depend>timerfd</depend>
+	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
@@ -161,8 +162,32 @@ static void timerfd_timer_ack(int handle, unsigned int quantity)
 {
 	uint64_t expirations;
 	int read_result = 0;
+	struct timerfd_timer *our_timer, find_helper = {
+		.handle = handle,
+	};
+
+	if (!(our_timer = ao2_find(timerfd_timers, &find_helper, OBJ_POINTER))) {
+		ast_log(LOG_ERROR, "Couldn't find a timer with handle %d\n", handle);
+		return;
+	}
+
+	ao2_lock(our_timer);
 
 	do {
+		struct itimerspec timer_status;
+
+		if (timerfd_gettime(handle, &timer_status)) {
+			ast_log(LOG_ERROR, "Call to timerfd_gettime() error: %s\n", strerror(errno));
+			expirations = 0;
+			break;
+		}
+
+		if (timer_status.it_value.tv_sec == 0 && timer_status.it_value.tv_nsec == 0) {
+			ast_debug(1, "Avoiding read on disarmed timerfd %d\n", handle);
+			expirations = 0;
+			break;
+		}
+
 		read_result = read(handle, &expirations, sizeof(expirations));
 		if (read_result == -1) {
 			if (errno == EINTR || errno == EAGAIN) {
@@ -174,6 +199,9 @@ static void timerfd_timer_ack(int handle, unsigned int quantity)
 		}
 	} while (read_result != sizeof(expirations));
 
+	ao2_unlock(our_timer);
+	ao2_ref(our_timer, -1);
+
 	if (expirations != quantity) {
 		ast_debug(2, "Expected to acknowledge %u ticks but got %llu instead\n", quantity, (unsigned long long) expirations);
 	}
